<?php
/**
 * Clamd - A ClamAV plugin for CakePHP
 * Copyright (C) 2009 Stichting Lone Wolves
 * Written by Sander Marechal <s.marechal@jejik.com>
 *
 * Licensed under the MIT License
 * Redistributions of files must retain the above copyright notice.
 *
 * @package clamd
 * @copyright Copyright (C) 2009 Stichting Lone Wolves
 * @license http://www.opensource.org/licenses/mit-license.php The MIT License
 */

App::import('Core', 'clamd.Clamd');

/**
 * A simple shell to interface with Clamd
 */
class ClamdShell extends Shell
{
	/** @var object Reference to the Clamd object */
	private $clamd = null;

	/**
	 * Print a welcome message
	 */
	public function initialize() {
		$this->clamd = null;
		$this->out('Cake Clamd Shell. Type "help" for usage information.');
		$this->hr();
	}

	/**
	 * Main shell execution loop
	 */
	public function main()
	{
		while (true) {
			$text = $this->in('');

			if (substr($text, 0, 7) == 'connect') {
				sscanf($text, '%s %s %d %d', $command, $host, $port, $timeout);
				$this->clamd = new Clamd($host, $port, $timeout);
				continue;
			}

			if ($text == 'quit' || $text == 'exit' || $text == 'q') {
				$this->_stop();
			}

			if (substr($text, 0, 4) == 'help') {
				$this->help();
				continue;
			}

			if ($this->clamd == null) {
				$this->out('Not connected. Connect to a ClamAV daemon first');
				continue;
			}

			if ($text == 'ping') {
				if (!$response = $this->clamd->ping()) {
					$this->out('Error: ' . $this->clamd->lastError());
				} else {
					$this->out('PONG');
				}

				continue;
			}

			if (substr($text, 0, 4) == 'scan') {
				$msg = '';
				$file = substr($text, 5);
				$this->clamd->clearLastError();
				$response = $this->clamd->scan($file, $msg);

				if ($response === false) {
					$this->out('Error: ' . $this->clamd->lastError());
				} elseif ($response == Clamd::OK) {
					$this->out($file . ' is clean');
				} elseif ($response == Clamd::FOUND) {
					$this->out($file . ' is infected with ' . $msg);
				} else {
					$this->out('Error: ' . $msg);
				}
				continue;
			}

			if (substr($text, 0, 5) == 'rscan') {
				$file = substr($text, 6);
				$this->clamd->clearLastError();
				if (!$response = $this->clamd->rscan($file, true)) {
					$this->out('Error: ' . $this->clamd->lastError());
					continue;
				}

				foreach ($response as $file) {
					if ($file['status'] == Clamd::FOUND) {
						$this->out($file['file'] . ' is infected with ' . $file['msg']);
					} else {
						$this->out($file['file'] . ' error: ' . $file['msg']);
					}
				}
				continue;
			}

			$this->out('Unknown command: ' . $text);
		}
	}

	/**
	 * Print shell help
	 */
	public function help()
	{
		$this->out('Interactive commandline interface to the Clamd plugin');
		$this->hr();
		$this->out("Usage: cake clamd");
		$this->hr();
		$this->out('Commands:');
		$this->out("\n\texit|quit|q\n\t\tQuit the shell");
		$this->out("\n\tconnect <host> [<port> [<timeout>]]\n\t\tConnect to a ClamAV daemon");
		$this->out("\n\tping\n\t\tSend a PING command to the clamav daemon");
		$this->out("\n\tscan <file>\n\t\tScan <file> for viruses");
		$this->out("\n\trscan <directory>\n\t\tRecursively scan <directory> for viruses. Only infected files and errors are returned.");
		$this->out("\n\thelp\n\t\tShow this help");
		$this->out('');
	}
}

?>
